const { Context, Endpoint } = require('bitsensor-node-js-core').Constants;

/**
 * HTTP Handler
 * @class HttpHandler
 */
class HttpHandler {
  /**
   * @param {Object} req
   * @param {Object} res
   * @param {Datapoint} datapoint
   */
  handle(req, res, datapoint) {
    datapoint.context[Context.IP] = (req.connection && req.connection.socket)
      ? req.connection.socket.remoteAddress
      : req.connection.remoteAddress || req.socket.remoteAddress;
    datapoint.context[Context.PROTOCOL_VERSION] = `HTTP/${req.httpVersion}`;
    datapoint.context[Context.QUERY_STRING] = require('url').parse(req.url).query;
    datapoint.context[Context.HTTP_USER_AGENT] = req.headers['user-agent'];
    datapoint.context[Context.HTTP_REFERER] = req.headers.referer;
    datapoint.context[Context.REQUEST_METHOD] = req.method;
    datapoint.context[Context.HTTP_ACCEPT] = req.headers.accept;
    datapoint.context[Context.HTTP_ACCEPT_CHARSET] = req.headers['accept-charset'];
    datapoint.context[Context.HTTP_ACCEPT_ENCODING] = req.headers['accept-encoding'];
    datapoint.context[Context.HTTP_ACCEPT_LANGUAGE] = req.headers['accept-language'];
    datapoint.context[Context.HTTPS] = (req.protocol === 'https').toString();
    datapoint.context[Context.X_FORWARDED_FOR] = req.headers['x-forwarded-for'];

    datapoint.endpoint[Endpoint.ADDRESS] = (req.connection && req.connection.socket)
      ? req.connection.socket.localAddress
      : req.connection.localAddress || req.socket.localAddress;
    datapoint.endpoint[Endpoint.NAME] = req.headers.host.split(':')[0];
    datapoint.endpoint[Endpoint.SOFTWARE] = 'Nodejs';
    datapoint.endpoint[Endpoint.PORT] = req.headers.host.split(':')[1];
    datapoint.endpoint[Endpoint.URI] = req.url;
    datapoint.endpoint[Endpoint.STATUS] = res.statusCode.toString();

    // Remove leading ::ffff: from context.ip and endpoint.address
    if (datapoint.context[Context.IP].substr(0, 7) == '::ffff:') {
      datapoint.context[Context.IP] = datapoint.context[Context.IP].substr(7)
    }
    if (datapoint.endpoint[Endpoint.ADDRESS].substr(0, 7) == '::ffff:') {
      datapoint.endpoint[Endpoint.ADDRESS] = datapoint.endpoint[Endpoint.ADDRESS].substr(7)
    }

    // TODO: duplicated in middleware parser
    // const query = require('url').parse(req.url, true).query;
    // for (const key in query) {
    //   datapoint.input[`http.${key}`] = query[key].toString();
    // }

    // Handles request cookies
    // TODO: test with signed cookies. Make cookie-parser compatible.
    req.headers && req.headers.cookie && req.headers.cookie.split(';').forEach(function (cookie) {
      const parts = cookie.match(/(.*?)=(.*)$/);
      if (!parts || !parts[1] || !parts[2])
        return;

      datapoint.input[`cookie.${parts[1].trim()}`] = (parts[2] || '').trim();
    });

    // Handles request body (works with body-parser middleware)
    // TODO: separate handler class. Removed one from middleware parser
    // req.body && Object.keys(req.body).forEach(key => datapoint.input[`http.body.${key}`] = req.body[key].toString())
  }
}

module.exports = HttpHandler;
