const HttpHandler = require('./http_handler');
/**
 * HTTP Handler Manager
 * @class HttpHandlerManager
 */
class HttpHandlerManager {
  /**
   * Creates an instance of HttpHandlerManager.
   * @memberof HttpHandlerManager
   */
  constructor() {
    this.preHandlers = [];
    this.postHandlers = [];
  }

  /**
   * @param {Object} request
   * @param {Object} response
   */
  preHandle(request, response) {
    const datapoint = bitSensor.getDatapoint();
    this.preHandlers.forEach((handler) => {
      handler.handle(request, response, datapoint);
    });
  }

  /**
   * @param {Object} request
   * @param {Object} response
   */
  postHandle(request, response) {
    const datapoint = bitSensor.getDatapoint();
    this.postHandlers.forEach((handler) => {
      handler.handle(request, response, datapoint);
    });
  }

  /**
   * @param {Handler} httpHandler
   */
  addPreHandler(httpHandler) {
    if (httpHandler instanceof HttpHandler) {
      this.preHandlers.push(httpHandler);
    }
  }

  /**
   * @param {Handler} httpHandler
   */
  addPostHandler(httpHandler) {
    if (httpHandler instanceof HttpHandler) {
      this.postHandlers.push(httpHandler);
    }
  }
}

module.exports = HttpHandlerManager;
