const {
  addCurrentToResult,
  recurseInArray,
  recurseInObject,
  checkEmptyAndPropNotNull,
} = require('../../core/json_flatten');
const { expect } = require('chai');

describe('Json unit', () => {
  describe('Add current to result', () => {
    it('add null to result', () => {
      const result = {};
      addCurrentToResult(result, 'test', null);
      expect(result.test.length === 0).to.be.true;
    });
    it('add current to result', () => {
      const result = {};
      addCurrentToResult(result, 'test', 'test1');
      expect(result.test === 'test1').to.be.true;
    });
  });
  describe('recurse in array', () => {
    it('Should be empty object', () => {
      const result = {};
      recurseInArray([], 'test', result);
      expect(Object.keys(result)).to.have.length(0);
    });
    it('Should flatten array', () => {
      const result = {};
      recurseInArray(['test1'], 'test', result);
      expect(result['test[0]'] === 'test1').to.be.true;
    });
  });
  describe('recurse in object', () => {
    it('isEmpty should be true', () => {
      const isEmpty = true;
      recurseInObject(null, null, isEmpty, {});
      expect(isEmpty).to.be.true;
    });
    it('isEmpty should be false', () => {
      let isEmpty = true;
      const result = {};
      isEmpty = recurseInObject({ test1: 'test1_value' }, 'test', isEmpty, result);
      expect(isEmpty).to.be.false;
    });
    it('result should be empty', () => {
      let isEmpty = true;
      const result = {};
      isEmpty = recurseInObject(null, null, isEmpty, result);
      expect(Object.keys(result)).to.have.length(0);
    });
    it('result should be flattened', () => {
      let isEmpty = true;
      const result = {};
      isEmpty = recurseInObject({ test1: 'test1_value' }, 'test', isEmpty, result);
      const expectedOutput = "{ 'test.test1': 'test1_value' }";
      expect(result.toString === expectedOutput.toString()).to.be.false;
    });
  });
  describe('Check empty and prop not null', () => {
    it('isEmpty false and prop null', () => {
      const result = {};
      checkEmptyAndPropNotNull(false, result, null);
      expect(Object.keys(result)).to.have.length(0);
    });
    it('isEmpty true and prop null', () => {
      const result = {};
      checkEmptyAndPropNotNull(true, result, null);
      expect(Object.keys(result)).to.have.length(0);
    });
    it('isEmpty false and prop not null', () => {
      const result = {};
      checkEmptyAndPropNotNull(false, result, 'test');
      expect(Object.keys(result)).to.have.length(0);
    });
    it('isEmpty true and prop not null adds to result', () => {
      const result = {};
      checkEmptyAndPropNotNull(true, result, 'test');
      expect(result.test != null).to.be.true;
    });
  });
});
