const { server, datapointCollector, core } = require('./app');
const errorToDatapoint = require('../core/error_to_datapoint');
const chai = require('chai');
const chaiHttp = require('chai-http');

const { expect } = chai;
chai.use(chaiHttp);

describe('middleware error test', () => {
  after((done) => {
    server.close();
    done();
  });
  it('response should have error', (done) => {
    require('./init');
    chai.request('http://localhost:3001')
      .get('/errortest')
      .end((err, res) => {
        expect(res.statusCode).to.not.equal;
        done(); // <= Call done to signal callback end
      });
  });
  it('datapoint should have error', (done) => {
    require('./init');
    bitSensor.runSession(() => {
      errorToDatapoint(new Error('error thrown'), () => { });
      expect(bitSensor.getDatapoint().errors.length).to.be.above(0);
      done(); // <= Call done to signal callback end
    });
  });
  it('datapoint error should contain generatedby plugin', (done) => {
    require('./init');
    bitSensor.runSession(() => {
      errorToDatapoint(new Error('error thrown'), () => { });
      expect(bitSensor.getDatapoint().errors[0].generatedby).to.be.equal(core.GeneratedBy.PLUGIN);
      done(); // <= Call done to signal callback end
    });
  });
  it('datapoint should have parsed request', (done) => { // <= Pass in done callback
    require('./init');
    chai.request('http://localhost:3001')
      .post('/posttest')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send({ myparam: 'test' })
      .end((err, res) => {
        expect(datapointCollector.length).to.be.above(0);
        done();
      });
  });
});
